class SettingsForm {
    async init() {
        this.resetApiState();

        await Localization.load(navigator.language.substring(0, 2), 'de');
        Localization.updateDocument(document);
        $('save').value = loca('settings.save');
        $('title').innerHTML = loca('settings.title').assign('<highlight>', '<span>').assign('</highlight>', '</span>');

        this.loadSettings();

        // reload data when document gets active
        let lastDocumentState = false;
        window.setInterval(() => {
            if (lastDocumentState == true && document.hidden == false) {
                this.loadSettings();
            }
            lastDocumentState = document.hidden;
        }, 500);


        $('settings_form').addEventListener('change', () => {
            this.enableSaveButton(true);
        });

        $('apiendpoint-use-credentials').addEventListener('click', () => {
            if ($('apiendpoint-use-credentials').checked) {
                this.enableAuthentification(true);
            } else {
                this.enableAuthentification(false);
            }

            const credentials = this.getCredentials();
            const api = this.getApiURL();
            let loadPropSets = true;
            if (!api) loadPropSets = false;
            if (!credentials) loadPropSets = false;
            if (credentials && credentials.username == '') loadPropSets = false;
            if (credentials && credentials.password == '') loadPropSets = false;

            if (loadPropSets) {
            this.loadPropSets(this.getApiURL(), this.getCredentials());
            }
        });

        $('apiEndpoint').addEventListener('change', () => {        
            // when user entered credentials remove them and put them in username/password field
            var url = parseURL($('apiEndpoint').value);
            if (url.username != '' && url.password != '') {
                $('apiEndpoint-username').value = url.username;
                $('apiEndpoint-password').value = url.password;
                this.enableAuthentification(true);
                $('apiEndpoint').value = removeCredentialsFromUrl($('apiEndpoint').value);
            }            

            this.loadPropSets(this.getApiURL(), this.getCredentials());
        });

        $('apiEndpoint-username').addEventListener('change', () => {
            this.loadPropSets(this.getApiURL(), this.getCredentials());
        });

        $('apiEndpoint-password').addEventListener('change', () => {
            this.loadPropSets(this.getApiURL(), this.getCredentials());
        });

        $('tab-basic').addEventListener('click', () => {
            $('tab-basic').classList.add('active');
            $('tab-dictionaries').classList.remove('active');

            $('settings_form').classList.remove('hidden');
            $('settings_dictionaries').classList.add('hidden');
        });

        $('tab-dictionaries').addEventListener('click', () => {
            $('tab-dictionaries').classList.add('active');
            $('tab-basic').classList.remove('active');

            $('settings_form').classList.add('hidden');
            $('settings_dictionaries').classList.remove('hidden');
        });

        // Save Settings
        $('save').addEventListener('click', () => {
            const settings = {};

            for (let key in Settings.state) {
                settings[key] = Settings.state[key];
                const el = $(key);
                if (el) {
                    if (el.type == 'checkbox') {
                        settings[key] = el.checked;
                    } else {
                        let val = el.value;
                        if (key === 'messageLevel' || key === 'spellcheckingLevel') {
                            const intVal = parseInt(val);
                            val = isNaN(intVal) ? null : intVal;
                        } else if (key === 'apiEndpoint') {
                            val = this.getApiURL();
                        }
                        
                        if (val === '') {
                            val = null;
                        }

                        settings[key] = val;
                    }
                }
            }

            settings.apiCredentials = this.getCredentials();

            ['styleForeign', 'styleOld', 'styleRegional', 'styleColloquial', 'styleFiller'].forEach(element => {
                let val = getRadioVal(element);
                if (val === '') val = null;
                if (val === 'true') val = true;
                if (val === 'false') val = false;
                settings[element] = val;
            });

            let dictionaries = [];

            document.querySelectorAll('[name="dictionary-item"]').forEach((item) => {
                if (item.checked) {
                    dictionaries.push(item.getAttribute('data-name'));
                }
            })

            settings['dictionaries'] = dictionaries;

            Settings.save(settings, () => {
                const status = $('status');
                if (status) {
                    status.textContent = loca('settings.optionsSaved');
                    status.style.opacity = 1;
                    this.enableSaveButton(false);
                    setTimeout(() => {
                        status.style.opacity = 0;

                        // we remove the activated entry, because we won't
                        // this to update
                        settings.activated = undefined;

                        // send changes to all open tabs
                        chrome.tabs.query({}, (tabs) => {
                            for (let i = 0; i < tabs.length; ++i) {
                                const message = { subject: 'update_settings', settings: settings };
                                chrome.tabs.sendMessage(tabs[i].id, message);
                            }
                        });
                    }, 1500);
                }
            });
       });
    }

    enableAuthentification(enable) {
        if (enable) {
            $('credentials-form').style.maxHeight = '210px';
            $('apiendpoint-use-credentials').checked = true;
        } else {
            $('credentials-form').style.maxHeight = '45px';
            $('apiendpoint-use-credentials').checked = false;
        }
    }

    getApiURL() {
        var url = fixDKSUrl($('apiEndpoint').value);
        return url;
    }

    getCredentials() {
        if ($('apiendpoint-use-credentials').checked) {
            return {
                username: $('apiEndpoint-username').value,
                password: $('apiEndpoint-password').value
            }
        };

        return null;
    }

    resetApiState() {
        this.apiState = {
            valid: false,
            dictionaries: null,
            propsets: null
        }
    }

    enableSaveButton(enabled) {
        if (enabled) {
            $('save').style.opacity = 1.0;
            $('save').style.pointerEvents = '';
        } else {
            $('save').style.opacity = 0.4;
            $('save').style.pointerEvents = 'none';
        }
    }
    
    updateUpdateMessage() {
        if (this.apiState.valid && this.apiState.dictionaries != null && this.apiState.propsets != null) {
            if (!this.apiState.dictionaries || !this.apiState.propsets) {
                $('update-message').classList.remove('hidden');
                return;
            }
        }
 
        $('update-message').classList.add('hidden');
    };

    updateUI() {
        // Update Dictionaries
        if (!Settings.dictionaries || Object.keys(Settings.dictionaries).length == 0) {
            $('dictionary_items').innerHTML = $('no_dictionaries_template').textContent;
            Localization.updateDocument($('dictionary_items'));
            return;
        }

        const tpl = $('dictionary_item_template').textContent;
        let html = '';
        let index = 0;

        for (var name in Settings.dictionaries) {
            if (Settings.dictionaries.hasOwnProperty(name)) {
                let itemHtml = tpl;
                itemHtml = itemHtml.assign('{index}', index++);
                itemHtml = itemHtml.assign('{name}', name);
                itemHtml = itemHtml.assign('{description}', Settings.dictionaries[name]);
                html += itemHtml;
            }
        }

        $('dictionary_items').innerHTML = html;

        $('dictionary_items').querySelectorAll('input').forEach((element) => {
            element.addEventListener('change', () => {
                this.enableSaveButton(true);
            })
            if (Settings.state.dictionaries.indexOf(element.getAttribute('data-name')) > -1) {
                element.checked = true;
            }
        });
        
        // Update Properties
        const propertySets = Settings.state.propertySets ? Settings.state.propertySets.split(',').map((item) => { return item.trim() }): [];

        var overwritteProperties = {
            'checklevel': null,
            'language': null,
            'orthstd': null,
            'stycolloquial': null,
            'styfiller': null,
            'styforeign': null,
            'styold': null,
            'styregional': null,
            'dictionaries': []
        };

        if (typeof (Settings.state.registeredPropertySets) !== "undefined") {
            for (var propSetKey in propertySets) {
                var name = propertySets[propSetKey].trim();
                var propertySet = Settings.state.registeredPropertySets[name];
                if (typeof(propertySet) !== 'undefined') {
                    if ((typeof(propertySet.enforce) !== 'undefined') && propertySet.enforce === true) {
                        if (typeof(propertySet.checklevel)      !== 'undefined') overwritteProperties['checklevel'] = propertySet.checklevel;
                        if (typeof(propertySet.language)        !== 'undefined') overwritteProperties['language'] = propertySet.language;
                        if (typeof(propertySet.orthstd)         !== 'undefined') overwritteProperties['orthstd'] = propertySet.orthstd;
                        if (typeof(propertySet.stycolloquial)   !== 'undefined') overwritteProperties['stycolloquial'] = propertySet.stycolloquial;
                        if (typeof(propertySet.styfiller)       !== 'undefined') overwritteProperties['styfiller'] = propertySet.styfiller;
                        if (typeof(propertySet.styforeign)      !== 'undefined') overwritteProperties['styforeign'] = propertySet.styforeign;
                        if (typeof(propertySet.styold)          !== 'undefined') overwritteProperties['styold'] = propertySet.styold;
                        if (typeof(propertySet.styregional)     !== 'undefined') overwritteProperties['styregional'] = propertySet.styregional;
                    }

                    if (typeof (propertySet.dictionaries) !== 'undefined') {
                        propertySet.dictionaries.forEach((item) => {
                            overwritteProperties['dictionaries'].push(item);
                        })
                    }
                }
            }
        }

        const checkElement = (id, property, defaultValue) => {
            var element = $('#' + id);
            if (property !== null) {
                element.classList.add('disabled');
                element.checked = property;
                selectByValue(id, property);
            } else {
                element.classList.remove('disabled');
                element.checked = defaultValue;
                selectByValue(id, defaultValue);
            }
        }

        const checkRadioElement = (id, property, defaultValue) => {
            let elements = getRadioNodes(id);
            for (var i = 0, len = elements.length; i < len; i++) {
                let element = elements[i];

                let radioValue = null;
                if (element.value === "true") radioValue = true;
                if (element.value === "false") radioValue = false;

                let val = property;
                if (property !== null) {
                    element.parentNode.classList.add('disabled');
                } else {
                    element.parentNode.classList.remove('disabled');
                    val = defaultValue;
                }

                if (val === radioValue) {
                    element.checked = true;
                } else {
                    element.checked = false;
                }
            }
        }

        checkElement('textLanguage', overwritteProperties.language, Settings.state.textLanguage)
        checkElement('spellcheckingLevel', overwritteProperties.checklevel, Settings.state.spellcheckingLevel)
        checkElement('orthographyStandard', overwritteProperties.orthstd, Settings.state.orthographyStandard)
        checkRadioElement('styleColloquial', overwritteProperties.stycolloquial, Settings.state.styleColloquial);
        checkRadioElement('styleFiller', overwritteProperties.styfiller, Settings.state.styleFiller);
        checkRadioElement('styleForeign', overwritteProperties.styforeign, Settings.state.styleForeign);
        checkRadioElement('styleOld', overwritteProperties.styold, Settings.state.styleOld);
        checkRadioElement('styleRegional', overwritteProperties.styregional, Settings.state.styleRegional);

        // Update Forced Dictionaries
        overwritteProperties.dictionaries.forEach((item) => {
            $('dictionary_items').querySelectorAll('input').forEach((element) => {
                if (element.getAttribute('data-name') === item) {
                    element.checked = true;
                    element.parentNode.classList.add('disabled');
                }
            });
        })
    }

    getDictionaries(apiEndpoint, credentials) {
        json_request('GET', apiEndpoint + '/dictionaries-with-description', credentials, (result) => {
            if (typeof (result) !== "undefined") {
                this.apiState.dictionaries = true;
                this.updateUpdateMessage();
                Settings.dictionaries = result;
                this.updateUI();
                this.updateUpdateMessage();
            } else {
                console.warn("Couldn't load dictionaries");
                this.apiState.dictionaries = false;
                this.updateUpdateMessage();
            }
        }, (error) => {
            console.warn("Couldn't load dictionaries");
            Settings.dictionaries = {};
            this.updateUI();
            this.apiState.dictionaries = false;
            this.updateUpdateMessage();
        });
    }

    checkVersion(apiEndpoint, credentials, next) {
        checkDKSVersion(apiEndpoint, credentials)
            .then((result) => {
                if (typeof (result) !== "undefined") {
                    this.apiState.valid = true;
                    $('apiEndpoint').classList.remove('error');
                } else {
                    this.apiState.valid = false;
                    $('apiEndpoint').classList.add('error');
                }
                next();
            })
            .catch((error) => {
                this.apiState.valid = false;
                $('apiEndpoint').classList.add('error');
                next();
            });
    }

    loadPropSets(apiEndpoint, credentials) {
        this.resetApiState();

        this.checkVersion(apiEndpoint, credentials, () => {
            json_request('GET', apiEndpoint + '/propset', credentials, (result) => {
                Settings.state.registeredPropertySets = {};
                if (typeof (result) !== "undefined") {
                    this.apiState.propsets = true;
                    this.updateUpdateMessage();
                    for (var key in result) {
                        var propSet = result[key];
                        Settings.state.registeredPropertySets[propSet.name] = propSet;
                    }

                    this.updateUI();
                } else {
                    this.apiState.propsets = false;
                    this.updateUpdateMessage();
                    console.warn("Couldn't load property sets. Please update DKS Version");
            }
            }, (errorRequest) => {
                Settings.state.registeredPropertySets = {};
                    if (errorRequest.status == 404) {
                    this.apiState.propsets = false;
                    this.updateUpdateMessage();
                    console.warn("Couldn't load property sets. Please update DKS Version");
                }
            });

            this.getDictionaries(apiEndpoint, credentials);
        });
    }

    loadSettings() {
        Settings.load((settings) => {
            this.loadPropSets(settings.apiEndpoint, settings.apiCredentials);

            this.enableSaveButton(false);

            for (let key in settings) {
                const el = $(key);
                if (el) {
                    if (el.type == 'checkbox') {
                        el.checked = settings[key];
                    } else {
                        if (typeof (settings[key]) === 'undefined' || settings[key] === null) {
                        } else {
                            if (key === 'apiEndpoint' && settings[key].trim() != '') {

                                var url = parseURL(settings[key]);

                                if (url.username != '' && url.password != '') {
                                    settings.apiCredentials = {
                                        username: url.username,
                                        password: url.password
                                    };

                                    el.value = removeCredentialsFromUrl(settings[key]);
                                } else {
                                    el.value = settings[key];
                                }
                            } else {
                                el.value = settings[key];
                            }
                        }
                    }
                }
            }

            if (settings.apiCredentials) {
                this.enableAuthentification(true);
                $('apiEndpoint-username').value = settings.apiCredentials.username;
                $('apiEndpoint-password').value = settings.apiCredentials.password;
            } else {
                this.enableAuthentification(false);
            }
        });
    }
}

document.addEventListener('DOMContentLoaded', () => {
    (new SettingsForm()).init();
});
